@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 module Cri
   # Cri::Command represents a command that can be executed on the command line.
   # It is also used for the command-line tool itself.
@@ -7,7 +9,7 @@ module Cri
     # command, is found.
     #
     # @api private
-    class OptionParserPartitioningDelegate
+    class ParserPartitioningDelegate
       # Returns the last parsed argument, which, in this case, will be the
       # first argument, which will be either nil or the command name.
       #
@@ -16,25 +18,25 @@ module Cri
 
       # Called when an option is parsed.
       #
-      # @param [Symbol] key The option key (derived from the long format)
+      # @param [Symbol] _key The option key (derived from the long format)
       #
-      # @param value The option value
+      # @param _value The option value
       #
-      # @param [Cri::OptionParser] option_parser The option parser
+      # @param [Cri::Parser] _parser The option parser
       #
       # @return [void]
-      def option_added(_key, _value, _option_parser); end
+      def option_added(_key, _value, _parser); end
 
       # Called when an argument is parsed.
       #
       # @param [String] argument The argument
       #
-      # @param [Cri::OptionParser] option_parser The option parser
+      # @param [Cri::Parser] parser The option parser
       #
       # @return [void]
-      def argument_added(argument, option_parser)
+      def argument_added(argument, parser)
         @last_argument = argument
-        option_parser.stop
+        parser.stop
       end
     end
 
@@ -85,9 +87,16 @@ module Cri
     attr_accessor :hidden
     alias hidden? hidden
 
-    # @return [Array<Hash>] The list of option definitions
+    # @return [Array<Cri::OptionDefinition>] The list of option definitions
     attr_accessor :option_definitions
 
+    # @return [Array<Hash>] The list of parameter definitions
+    attr_accessor :parameter_definitions
+
+    # @return [Boolean] Whether or not this command has parameters
+    attr_accessor :explicitly_no_params
+    alias explicitly_no_params? explicitly_no_params
+
     # @return [Proc] The block that should be executed when invoking this
     #   command (ignored for commands with subcommands)
     attr_accessor :block
@@ -123,6 +132,22 @@ module Cri
       dsl.command
     end
 
+    # Creates a new command using a DSL, from code defined in the given filename.
+    #
+    # @param [String] filename The filename that contains the command definition
+    #   as a string
+    #
+    # @return [Cri::Command] The newly defined command
+    def self.load_file(filename, infer_name: false)
+      code = File.read(filename, encoding: 'UTF-8')
+      define(code, filename).tap do |cmd|
+        if infer_name
+          command_name = File.basename(filename, '.rb')
+          cmd.modify { name command_name }
+        end
+      end
+    end
+
     # Returns a new command that has support for the `-h`/`--help` option and
     # also has a `help` subcommand. It is intended to be modified (adding
     # name, summary, description, other subcommands, …)
@@ -145,6 +170,8 @@ module Cri
       @aliases            = Set.new
       @commands           = Set.new
       @option_definitions = Set.new
+      @parameter_definitions = []
+      @explicitly_no_params = false
       @default_subcommand_name = nil
     end
 
@@ -165,8 +192,8 @@ module Cri
       self
     end
 
-    # @return [Hash] The option definitions for the command itself and all its
-    #   ancestors
+    # @return [Enumerable<Cri::OptionDefinition>] The option definitions for the
+    #   command itself and all its ancestors
     def global_option_definitions
       res = Set.new
       res.merge(option_definitions)
@@ -284,7 +311,7 @@ module Cri
         return if subcommand.nil?
 
         # Run
-        subcommand.run(opts_and_args_after_subcmd, opts_before_subcmd, hard_exit: hard_exit)
+        subcommand.run(opts_and_args_after_subcmd, parent_opts.merge(opts_before_subcmd), hard_exit: hard_exit)
       end
     rescue CriExitException => e
       exit(e.error? ? 1 : 0) if hard_exit
@@ -309,16 +336,19 @@ module Cri
         global_opts = parent_opts
       else
         # Parse
-        parser = Cri::OptionParser.new(
-          opts_and_args, global_option_definitions
+        parser = Cri::Parser.new(
+          opts_and_args,
+          global_option_definitions,
+          parameter_definitions,
+          explicitly_no_params?,
         )
-        handle_parser_errors_while { parser.run }
+        handle_errors_while { parser.run }
         local_opts  = parser.options
         global_opts = parent_opts.merge(parser.options)
-        args = parser.arguments
 
         # Handle options
         handle_options(local_opts)
+        args = handle_errors_while { parser.gen_argument_list }
       end
 
       # Execute
@@ -355,9 +385,9 @@ module Cri
 
     def handle_options(opts)
       opts.each_pair do |key, value|
-        opt_def = global_option_definitions.find { |o| (o[:long] || o[:short]) == key.to_s }
-        block = opt_def[:block]
-        block.call(value, self) if block
+        opt_defn = global_option_definitions.find { |o| (o.long || o.short) == key.to_s }
+        block = opt_defn.block
+        block&.call(value, self)
       end
     end
 
@@ -365,10 +395,15 @@ module Cri
       return [{}, opts_and_args.first, opts_and_args] if all_opts_as_args?
 
       # Parse
-      delegate = Cri::Command::OptionParserPartitioningDelegate.new
-      parser = Cri::OptionParser.new(opts_and_args, global_option_definitions)
+      delegate = Cri::Command::ParserPartitioningDelegate.new
+      parser = Cri::Parser.new(
+        opts_and_args,
+        global_option_definitions,
+        parameter_definitions,
+        explicitly_no_params?,
+      )
       parser.delegate = delegate
-      handle_parser_errors_while { parser.run }
+      handle_errors_while { parser.run }
 
       # Extract
       [
@@ -378,14 +413,20 @@ module Cri
       ]
     end
 
-    def handle_parser_errors_while
+    def handle_errors_while
       yield
-    rescue Cri::OptionParser::IllegalOptionError => e
-      warn "#{name}: illegal option -- #{e}"
+    rescue Cri::Parser::IllegalOptionError => e
+      warn "#{name}: unrecognised option -- #{e}"
       raise CriExitException.new(is_error: true)
-    rescue Cri::OptionParser::OptionRequiresAnArgumentError => e
+    rescue Cri::Parser::OptionRequiresAnArgumentError => e
       warn "#{name}: option requires an argument -- #{e}"
       raise CriExitException.new(is_error: true)
+    rescue Cri::Parser::IllegalOptionValueError => e
+      warn "#{name}: #{e.message}"
+      raise CriExitException.new(is_error: true)
+    rescue Cri::ArgumentList::ArgumentCountMismatchError => e
+      warn "#{name}: #{e.message}"
+      raise CriExitException.new(is_error: true)
     end
   end
 end
